@@ -2,7 +2,7 @@
 ######################################################################
 ### SNMP_util -- SNMP utilities using SNMP_Session.pm and BER.pm
 ######################################################################
-### Copyright (c) 1998-2001, Mike Mitchell.
+### Copyright (c) 1998-2002, Mike Mitchell.
 ###
 ### This program is free software; you can redistribute it under the
 ### "Artistic License" included in this distribution (file "Artistic").
@@ -20,6 +20,8 @@
 ### Tobias Oetiker <oetiker@ee.ethz.ch>: HASH as first OID to set SNMP options
 ### Simon Leinen <simon@switch.ch>: 'undefined port' bug
 ### Daniel McDonald <dmcdonald@digicontech.com>: request for getbulk support
+### Laurent Girod <girod.laurent@pmintl.ch>: code for snmpwalkhash
+### Ian Duplisse <i.duplisse@cablelabs.com>: MIB parsing suggestions
 ######################################################################
 
 package SNMP_util;
@@ -31,15 +33,15 @@ use vars qw(@ISA @EXPORT $VERSION);
 use Exporter;
 use Carp;
 
-use BER "0.86";
-use SNMP_Session "0.86";
+use BER "0.82";
+use SNMP_Session "0.83";
 use Socket;
 
-$VERSION = '0.86';
+$VERSION = '0.92';
 
 @ISA = qw(Exporter);
 
-@EXPORT = qw(snmpget snmpgetnext snmpwalk snmpset snmptrap snmpgetbulk snmpmaptable snmpmapOID snmpMIB_to_OID snmpLoad_OID_Cache snmpQueue_MIB_File);
+@EXPORT = qw(snmpget snmpgetnext snmpwalk snmpset snmptrap snmpgetbulk snmpmaptable snmpmaptable4 snmpwalkhash snmpmapOID snmpMIB_to_OID snmpLoad_OID_Cache snmpQueue_MIB_File);
 
 # The OID numbers from RFC1213 (MIB-II) and RFC1315 (Frame Relay)
 # are pre-loaded below.
@@ -54,13 +56,13 @@ $VERSION = '0.86';
     'mib-2' => '1.3.6.1.2.1',
     'system' => '1.3.6.1.2.1.1',
     'sysDescr' => '1.3.6.1.2.1.1.1.0',
-    'sysObjectID' => '1.3.6.1.2.1.1.2',
+    'sysObjectID' => '1.3.6.1.2.1.1.2.0',
     'sysUpTime' => '1.3.6.1.2.1.1.3.0',
     'sysUptime' => '1.3.6.1.2.1.1.3.0',
     'sysContact' => '1.3.6.1.2.1.1.4.0',
     'sysName' => '1.3.6.1.2.1.1.5.0',
     'sysLocation' => '1.3.6.1.2.1.1.6.0',
-    'sysServices' => '1.3.6.1.2.1.1.7',
+    'sysServices' => '1.3.6.1.2.1.1.7.0',
     'interfaces' => '1.3.6.1.2.1.2',
     'ifNumber' => '1.3.6.1.2.1.2.1.0',
     'ifTable' => '1.3.6.1.2.1.2.2',
@@ -318,6 +320,10 @@ $VERSION = '0.86';
     'enterprises' => '1.3.6.1.4.1',
   );
 
+# GIL
+my %revOIDS = ();	# Reversed %SNMP_util::OIDS hash
+my $RevNeeded = 1;
+
 my $agent_start_time = time;
 
 undef $SNMP_util::Host;
@@ -328,17 +334,20 @@ $SNMP_util::Debug = 0;
 $SNMP_util::CacheFile = "OID_cache.txt";
 $SNMP_util::CacheLoaded = 0;
 
-srand(time|$$);
+srand(time + $$);
 
 ### Prototypes
 sub snmpget ($@);
 sub snmpgetnext ($@);
 sub snmpopen ($$$);
 sub snmpwalk ($@);
+sub snmpwalk_flg ($$@);
 sub snmpset ($@);
 sub snmptrap ($$$$$@);
 sub snmpgetbulk ($$$@);
 sub snmpmaptable ($$@);
+sub snmpmaptable4 ($$$@);
+sub snmpwalkhash ($$@);
 sub toOID (@);
 sub snmpmapOID (@);
 sub snmpMIB_to_OID ($);
@@ -346,7 +355,6 @@ sub encode_oid_with_errmsg ($);
 sub Check_OID ($);
 sub snmpLoad_OID_Cache ($);
 sub snmpQueue_MIB_File (@);
-sub walk_fun ($@);
 
 sub version () { $VERSION; }
 
@@ -366,7 +374,6 @@ sub snmpopen ($$$) {
   ($host, $port, $timeout, $retries, $backoff, $version) = split(':', $host, 6)
     if ($host =~ /:/);
   $version = '1' unless defined $version;
-  undef($port) if (defined($port) && length($port) <= 0);
   if (defined($port) && ($port =~ /^([^!]*)!(.*)$/)) {
     ($port, $lhost) = ($1, $2);
     $nlhost = $lhost;
@@ -374,6 +381,7 @@ sub snmpopen ($$$) {
     undef($lhost) if (defined($lhost) && (length($lhost) <= 0));
     undef($lport) if (defined($lport) && (length($lport) <= 0));
   }
+  undef($port) if (defined($port) && length($port) <= 0);
   $port = 162 if ($type == 1 && !defined($port));
   $nhost = "$community\@$host";
   $nhost .= ":" . $port if (defined($port));
@@ -514,12 +522,26 @@ sub snmpgetnext ($@) {
 #
 sub snmpwalk ($@) {
   my($host, @vars) = @_;
+  return(&snmpwalk_flg($host, undef, @vars));
+}
+
+#
+# Walk the MIB, putting everything you find into hashes.
+#
+sub snmpwalkhash($$@) {
+#  my($host, $hash_sub, @vars) = @_;
+  return(&snmpwalk_flg( @_ ));
+}
+
+sub snmpwalk_flg ($$@) {
+  my($host, $hash_sub, @vars) = @_;
   my(@enoid, $var, $response, $bindings, $binding);
   my($value, $upoid, $oid, @retvals);
   my($got, @nnoid, $noid, $ok);
   my $session;
   my(%soid);
-
+  my(%done, %rethash);
+	
   $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
     carp "SNMPWALK Problem for $host\n"
@@ -530,6 +552,15 @@ sub snmpwalk ($@) {
   @enoid = toOID(@vars);
   return undef unless defined $enoid[0];
 
+  # GIL
+  #
+  # Create/Refresh a reversed hash with oid -> name
+  #
+  if (defined($hash_sub) && $RevNeeded) {
+    %revOIDS = reverse %SNMP_util::OIDS;
+    $RevNeeded = 0;
+  }
+
   $got = 0;
   @nnoid = @enoid;
   undef @vars;
@@ -562,18 +593,37 @@ sub snmpwalk ($@) {
       if ($ok) {
 	my $tmp = encode_oid_with_errmsg ($tempo);
 	return undef unless defined $tmp;
-	$soid{$upoid} = $tmp;
+	next if (exists($done{$tmp}));	# GIL
+	push @nnoid, $tmp;
 	my $tempv = pretty_print($value);
-	$tempo=~s/^$upoid\.//;
-	push @retvals, "$tempo:$tempv";
+	if (defined($hash_sub)) {
+	  #
+	  # extract name of the oid, if possible, the rest becomes the instance
+	  #
+	  my $inst;
+	  while (!exists($revOIDS{$tempo}) && length($tempo)) {
+	    $tempo =~ s/(\.\d+?)$//;
+	    $inst = $1.$inst;
+	  }	
+	  #
+	  # call hash_sub
+	  #
+	  &$hash_sub(\%rethash, $host, $revOIDS{$tempo}, $tempo, $inst, $tempv);
+	} else {
+	  $tempo=~s/^$upoid\.//;
+	  push @retvals, "$tempo:$tempv";
+	}
+	$done{$tmp} = 1;	# GIL
       }
     }
-    @nnoid = values(%soid);
-    undef %soid;
     last if ($#nnoid < 0);
   }
   if ($got) {
-    return (@retvals);
+    if (defined($hash_sub)) {
+    	return (%rethash)
+    } else {
+    	return (@retvals);
+    }
   } else {
     $var = join(' ', @vars);
     carp "SNMPWALK Problem for $var on $host\n"
@@ -739,11 +789,16 @@ sub snmpgetbulk ($$$@) {
 #
 sub snmpmaptable($$@) {
   my($host, $fun, @vars) = @_;
+  return snmpmaptable4($host, $fun, 0, @vars);
+}
+
+sub snmpmaptable4($$$@) {
+  my($host, $fun, $max_reps, @vars) = @_;
   my(@enoid, $var, $session);
 
   $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
-    carp "SNMPMAP_TABLE Problem for $host\n"
+    carp "SNMPMAPTABLE Problem for $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -752,19 +807,22 @@ sub snmpmaptable($$@) {
     push(@enoid, [split('\.', pretty_print($var))]);
   }
 
-  return $session->map_table([@enoid],
-			      sub() {
-				my ($ind, @vals) = @_;
-				my (@pvals, $val);
+  $max_reps = $session->default_max_repetitions() if ($max_reps <= 0);
 
-				foreach $val (@vals) {
-				  push(@pvals, pretty_print($val));
-				}
-				&$fun($ind, @pvals);
-			      }
-			      );
+  return $session->map_table_start_end( [@enoid],
+					sub() {
+					  my ($ind, @vals) = @_;
+					  my (@pvals, $val);
+
+					  foreach $val (@vals) {
+					    push(@pvals, pretty_print($val));
+					  }
+					  &$fun($ind, @pvals);
+					},
+					"", undef, $max_reps);
 }
 
+
 #
 #  Given an OID in either ASN.1 or mixed text/ASN.1 notation, return an
 #  encoded OID.
@@ -809,6 +867,10 @@ sub toOID(@) {
       unless ($SNMP_Session::suppress_warnings > 1);
       next;
     }
+    if ($var =~ /\"(.*)\"/) {
+      $tmp = sprintf("%d.%s", length($1), join(".", map(ord, split(//, $1))));
+      $var =~ s/\".*\"/$tmp/;
+    }
     print "toOID: $var\n" if $SNMP_util::Debug;
     $tmp = encode_oid_with_errmsg($var);
     return undef unless defined $tmp;
@@ -834,8 +896,10 @@ sub snmpmapOID(@)
     next unless($oid =~ /^((\d+.)*\d+)$/);
 
     $SNMP_util::OIDS{$txt} = $oid;
+    $RevNeeded = 1;
     print "snmpmapOID: $txt => $oid\n" if $SNMP_util::Debug;
   }
+  
   return undef;
 }
 
@@ -913,13 +977,13 @@ sub snmpQueue_MIB_File (@) {
 }
 
 #
-# Read in the passed list of MIB files, parsing them
+# Read in the passed MIB file, parsing it
 # for their text-to-OID mappings
 #
 sub snmpMIB_to_OID ($) {
   my($arg) = @_;
   my($quote, $buf, $var, $code, $val, $tmp, $tmpv, $strt);
-  my($ret);
+  my($ret, $pass, $pos, $need2pass, $cnt, %prev);
   my(%Link) = (
     'org' => 'iso',
     'dod' => 'org',
@@ -939,104 +1003,141 @@ sub snmpMIB_to_OID ($) {
   }
   print "snmpMIB_to_OID: loading $arg\n" if $SNMP_util::Debug;
   $ret = 0;
-  while(<MIB>) {
-    s/--.*--//g;	# throw away comments (-- anything --)
-    s/--.*//;		# throw away comments (-- anything EOL)
-    if ($quote) {
-      next unless /"/;
-      $quote = 0;
-    }
-    chop;
+  $pass = 0;
+  $need2pass = 1;
+  $cnt = 0;
+  $pos = tell(MIB);
+  while($need2pass) {
+    while(<MIB>) {
+      s/--.*--//g;	# throw away comments (-- anything --)
+      s/--.*//;		# throw away comments (-- anything EOL)
+      if ($quote) {
+	next unless /"/;
+	$quote = 0;
+      }
+      chop;
 #
 #	$buf = "$buf $_";
 # Previous line removed (and following replacement)
 # suggested by Brian Reichert, reichert@numachi.com
 #
-    $buf .= ' ' . $_;
-    $buf =~ s/\s+/ /g;
+      $buf .= ' ' . $_;
+      $buf =~ s/\s+/ /g;
 
-    if ($buf =~ / DEFINITIONS ::= BEGIN/) {
-      undef %Link;
-      %Link = (
-	'org' => 'iso',
-	'dod' => 'org',
-	'internet' => 'dod',
-	'directory' => 'internet',
-	'mgmt' => 'internet',
-	'mib-2' => 'mgmt',
-	'experimental' => 'internet',
-	'private' => 'internet',
-	'enterprises' => 'private',
-      );
-    }
+      if ($buf =~ / DEFINITIONS ::= BEGIN/) {
+	if ($pass == 0 && $need2pass) {
+	  seek(MIB, $pos, 0);
+	  $buf = "";
+	  $pass = 1;
+	  $need2pass = 0;
+	  $cnt = 0;
+	  next;
+	}
+	$need2pass = 0;
+	$pass = 0;
+	$pos = tell(MIB);
+	undef %Link;
+	undef %prev;
+	%Link = (
+	  'org' => 'iso',
+	  'dod' => 'org',
+	  'internet' => 'dod',
+	  'directory' => 'internet',
+	  'mgmt' => 'internet',
+	  'mib-2' => 'mgmt',
+	  'experimental' => 'internet',
+	  'private' => 'internet',
+	  'enterprises' => 'private',
+	);
+	$buf = "";
+	next;
+      }
 
-    $buf =~ s/OBJECT-TYPE/OBJECT IDENTIFIER/;
-    $buf =~ s/OBJECT-IDENTITY/OBJECT IDENTIFIER/;
-    $buf =~ s/MODULE-IDENTITY/OBJECT IDENTIFIER/;
-    $buf =~ s/ IMPORTS .*\;//;
-    $buf =~ s/ SEQUENCE {.*}//;
-    $buf =~ s/ SYNTAX .*//;
-    $buf =~ s/ [\w-]+ ::= OBJECT IDENTIFIER//;
-    $buf =~ s/ OBJECT IDENTIFIER .* ::= {/ OBJECT IDENTIFIER ::= {/;
-    $buf =~ s/".*"//;
-    if ($buf =~ /"/) {
-      $quote = 1;
-    }
+      $buf =~ s/OBJECT-TYPE/OBJECT IDENTIFIER/;
+      $buf =~ s/OBJECT-IDENTITY/OBJECT IDENTIFIER/;
+      $buf =~ s/OBJECT-GROUP/OBJECT IDENTIFIER/;
+      $buf =~ s/MODULE-IDENTITY/OBJECT IDENTIFIER/;
+      $buf =~ s/ IMPORTS .*\;//;
+      $buf =~ s/ SEQUENCE {.*}//;
+      $buf =~ s/ SYNTAX .*//;
+      $buf =~ s/ [\w-]+ ::= OBJECT IDENTIFIER//;
+      $buf =~ s/ OBJECT IDENTIFIER .* ::= {/ OBJECT IDENTIFIER ::= {/;
+      $buf =~ s/".*"//;
+      if ($buf =~ /"/) {
+	$quote = 1;
+      }
 
-    if ($buf =~ / ([\w\-]+) OBJECT IDENTIFIER ::= {([^}]+)}/) {
-      $var = $1;
-      $buf = $2;
-      undef $val;
-      $buf =~ s/ +$//;
-      ($code, $val) = split(' ', $buf, 2);
+      if ($buf =~ / ([\w\-]+) OBJECT IDENTIFIER ::= {([^}]+)}/) {
+	$var = $1;
+	$buf = $2;
+	undef $val;
+	$buf =~ s/ +$//;
+	($code, $val) = split(' ', $buf, 2);
 
-      if (!defined($val) || (length($val) <= 0)) {
-	$SNMP_util::OIDS{$var} = $code;
-	$ret++;
-	print "'$var' => '$code'\n" if $SNMP_util::Debug;
-      } else {
-	$strt = $code;
-	while($val =~ / /) {
-	  ($tmp, $val) = split(' ', $val, 2);
-	  if ($tmp =~ /([\w\-]+)\((\d+)\)/) {
-	    $tmp = $1;
-	    $tmpv = "$SNMP_util::OIDS{$strt}.$2";
-	    $Link{$tmp} = $strt;
-	    if (defined($SNMP_util::OIDS{$tmp})) {
-	      if ($tmpv ne $SNMP_util::OIDS{$tmp}) {
-		$strt = "$strt.$tmp";
-		$SNMP_util::OIDS{$strt} = $tmpv;
-		$ret++;
+	if (!defined($val) || (length($val) <= 0)) {
+	  $SNMP_util::OIDS{$var} = $code;
+	  $cnt++;
+	  print "'$var' => '$code'\n" if $SNMP_util::Debug;
+	} else {
+	  $strt = $code;
+	  while($val =~ / /) {
+	    ($tmp, $val) = split(' ', $val, 2);
+	    if ($tmp =~ /([\w\-]+)\((\d+)\)/) {
+	      $tmp = $1;
+	      $tmpv = "$SNMP_util::OIDS{$strt}.$2";
+	      $Link{$tmp} = $strt;
+	      if (!defined($prev{$tmp}) && defined($SNMP_util::OIDS{$tmp})) {
+		if ($tmpv ne $SNMP_util::OIDS{$tmp}) {
+		  $strt = "$strt.$tmp";
+		  $SNMP_util::OIDS{$strt} = $tmpv;
+		  $cnt++;
+		}
+	      } else {
+		$prev{$tmp} = 1;
+		$SNMP_util::OIDS{$tmp} = $tmpv;
+		$cnt++;
+		$strt = $tmp;
 	      }
-	    } else {
-	      $SNMP_util::OIDS{$tmp} = $tmpv;
-	      $ret++;
-	      $strt = $tmp;
 	    }
 	  }
-	}
 
-	if (!defined($SNMP_util::OIDS{$strt})) {
-	  carp "snmpMIB_to_OID: $arg: \"$strt\" prefix unknown, load the parent MIB first.\n"
-	    unless ($SNMP_Session::suppress_warnings > 1);
-	}
-	$Link{$var} = $strt;
-	$val = "$SNMP_util::OIDS{$strt}.$val";
-	if (defined($SNMP_util::OIDS{$var})) {
-	  if ($val ne $SNMP_util::OIDS{$var}) {
-	    $var = "$strt.$var";
+	  if (!defined($SNMP_util::OIDS{$strt})) {
+	    if ($pass) {
+	      carp "snmpMIB_to_OID: $arg: \"$strt\" prefix unknown, load the parent MIB first.\n"
+		unless ($SNMP_Session::suppress_warnings > 1);
+	    } else {
+		$need2pass = 1;
+	    }
 	  }
+	  $Link{$var} = $strt;
+	  $val = "$SNMP_util::OIDS{$strt}.$val";
+	  if (!defined($prev{$var}) && defined($SNMP_util::OIDS{$var})) {
+	    if ($val ne $SNMP_util::OIDS{$var}) {
+	      $var = "$strt.$var";
+	    }
+	  }
+
+	  $SNMP_util::OIDS{$var} = $val;
+	  $prev{$var} = 1;
+	  $cnt++;
+
+	  print "'$var' => '$val'\n" if $SNMP_util::Debug;
 	}
-
-	$SNMP_util::OIDS{$var} = $val;
-	$ret++;
-
-	print "'$var' => '$val'\n" if $SNMP_util::Debug;
+	undef $buf;
       }
-      undef $buf;
+    }
+    if ($pass == 0 && $need2pass) {
+      seek(MIB, $pos, 0);
+      $buf = "";
+      $pass = 1;
+      $cnt = 0;
+    } else {
+      $ret += $cnt;
+      $need2pass = 0;
     }
   }
   close(MIB);
+  $RevNeeded = 1;
   return $ret;
 }
 
