@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.19  -- Rateup
+ MRTG 2.9.22  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -61,7 +61,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.19";
+char *VERSION = "2.9.22";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -332,23 +332,23 @@ int peak;
 	avmxout /= valsamp;
     }
 	
-    printf("%lu\n",(unsigned long)(maxin/muli));
-    printf("%lu\n",(unsigned long)(maxout/muli));
+    printf("%lu\n",(unsigned long)(maxin/muli+.5));
+    printf("%lu\n",(unsigned long)(maxout/muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(maxpercent));
+       printf("%lu\n",(unsigned long)(maxpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(avin/muli));
-    printf("%lu\n",(unsigned long)(avout/muli));
+    printf("%lu\n",(unsigned long)(avin/muli+.5));
+    printf("%lu\n",(unsigned long)(avout/muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(avpercent));
+       printf("%lu\n",(unsigned long)(avpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(latestin/muli));
-    printf("%lu\n",(unsigned long)(latestout/muli));
+    printf("%lu\n",(unsigned long)(latestin/muli+.5));
+    printf("%lu\n",(unsigned long)(latestout/muli+.5));
     if (options & OPTION_DORELPERCENT) {
-       printf("%lu\n",(unsigned long)(latestpercent));
+       printf("%lu\n",(unsigned long)(latestpercent+.5));
     }
-    printf("%lu\n",(unsigned long)(avmxin/muli));
-    printf("%lu\n",(unsigned long)(avmxout/muli));
+    printf("%lu\n",(unsigned long)(avmxin/muli+.5));
+    printf("%lu\n",(unsigned long)(avmxout/muli+.5));
     
     if (maxv < 0 || (unsigned long) maxv < maxs) {maxv = maxs;}
       
@@ -987,17 +987,17 @@ int absupdate;
 	}
 	/* gauge option */
 	if (absupdate == 2) {
-	    inrate = diff(in,last.in);
-	    outrate = diff(out,last.out);
+	    inrate = diff(in,last.in)+.5;
+	    outrate = diff(out,last.out)+.5;
         } else if (absupdate == 3) {
-            inrate=diff(in,last.in)*(3600.0/(period*1.0));
-            outrate=diff(out,last.out)*(3600.0/(period*1.0));
+            inrate=diff(in,last.in)*(3600.0/(period*1.0))+.5;
+            outrate=diff(out,last.out)*(3600.0/(period*1.0))+.5;
         } else if (absupdate == 4)  {
-            inrate=diff(in,last.in)*(60.0/(period*1.0));
-            outrate=diff(out,last.out)*(60.0/(period*1.0));
+            inrate=diff(in,last.in)*(60.0/(period*1.0))+.5;
+            outrate=diff(out,last.out)*(60.0/(period*1.0))+.5;
 	} else {
-	    inrate = diff(in,last.in) / period;
-	    outrate = diff(out,last.out) / period;
+	    inrate = diff(in,last.in) / period + .5;
+	    outrate = diff(out,last.out) / period + .5;
 	}
     }
     if (inrate < 0 || (unsigned) inrate > abs_max) {
